import boto3
import logging

class RGBTableHelper:
    def __init__(self):
        self.client = boto3.client('dynamodb')
        self.table_name = 'ColorSplashRGB'
        self.rgb_attribute_key = 'RGB'
        self.imageids_attribute_key = 'ImageIds'
        self.all_keys = 'RGB, ImageIds'

    def update_rgb_values(self, image_dict):
        '''Given a dictionary of RGB -> ImageIds, UPDATE the RGB row in DynamoDB with the given
        ImageIds'''

        for key, value in image_dict.items():
            rgb_ids = set()
            try:
                rgb_ids = self.get_key(key)
            except KeyError as e:
                logging.debug('Key %s does not exist in %s table. Creating row...', key, self.table_name)
                pass

            rgb_ids |= value # Set join

            # If putting the key fails, I want to fail the entire update so we don't lose progress. 
            # Since we do a set join we dedupe values already updated on the next attempt
            self.put_key(key, rgb_ids)
                

    def get_key(self, key):
        '''Given a RGB key, retrieve the ImageIds attribute and deserialize it into native 
        python types'''

        if key is None:
            raise TypeError("Parameter 'key' cannot be of type None")

        try :
            response = self.client.get_item(
                Key={
                    self.rgb_attribute_key: {
                        'S': key
                    }
                },
                TableName=self.table_name
            )

            if 'Item' not in response:
                raise KeyError("No such key: " + key)
            else:
                return self.deserialize_imageids_attribute(response['Item'][self.imageids_attribute_key])
        except Exception as e:
            logging.error(e)
            raise

    def put_key(self, key, value):
        ''' Given a string RGB Key and a list of strings, serialize the python native types to 
        DynamoDB acceptable structures and insert them into the database'''
        if key is None or value is None:
            raise TypeError("Parameter key or value is None. (Key, value) -> (%s, %s)", key, value)

        try:
            return self.client.put_item(
                TableName=self.table_name,
                Item={
                    'RGB': {
                        'S': key
                    },
                    'ImageIds': self.serialize_imageids_attribute(value)
                }
            )
        except Exception as e:
            logging.error(e)
            raise

    def scan_rgbs(self):
        '''Most common use case of the table is to scan in all of the primary keys. This method
        builds on top of the scan method to only require the single primary key attribute to be 
        returned using ProjectionExpressions.'''
        items = self.scan(self.rgb_attribute_key)
        return items.keys()

    def scan(self, required_attributes=''):
        '''The scan operation will scan the entire table and return it in a dictionary with RGB
        attribute as the key and ImageIds as the value'''
        if not required_attributes:
            required_attributes = self.all_keys

        try : 
            response = self.client.scan(
                TableName=self.table_name,
                ProjectionExpression=required_attributes
            )

            return self.deserialize_color_splash_rgb_items(response['Items'])
        except Exception as e:
            logging.error("Error when scanning %s table", self.table_name, exc_info=True)

    def deserialize_color_splash_rgb_items(self, serialized_items):
        ''' Given a row of the table, deserialize the DynamoDB specific format into python native types'''
        items = {}
        for item in serialized_items:
            if self.rgb_attribute_key in item:
                rgb_attribute = self.deserialize_rgb_attribute(item[self.rgb_attribute_key])
                if self.imageids_attribute_key in item:
                    imageids_attribute = self.deserialize_imageids_attribute(item[self.imageids_attribute_key])
                    items[rgb_attribute] = imageids_attribute
                else:
                    items[rgb_attribute] = None

        return items

    def deserialize_rgb_attribute(self, serialized_data):
        '''Given the dynamoDB native format of the RGB attribute, deserialize it into its 
        native python type. Example input -> {'S': '[114.0, 116.0, 101.0]'}'''
        if 'S' not in serialized_data:
            raise KeyError('No such string key in RGB attribute')
        
        return serialized_data['S']

    def deserialize_imageids_attribute(self, serialized_data):
        '''Given the dynamoDB native format of the ImageIds attribute, deserialize it into its 
        native python type. Example input -> {'L': [{'S': 'WvlS1yWAu8c', 'S': 'zsgS1yxAu8c'}]}'''
        id_set = set()
        if 'L' not in serialized_data:
            raise KeyError('No such list key in serialized data provided')

        for item in serialized_data['L']:
            id_set.add(item['S']) 

        return id_set

    def serialize_imageids_attribute(self, unserialized_string_list_data):
        '''Given native python types, serialize it into the format expected for the ImageIds 
        attribute in DynamoDB'''
        l = []
        for item in unserialized_string_list_data:
            l.append({'S': item})
        return {'L': l}



import boto3
import logging

class ImageIdsTableHelper:
    def __init__(self):
        self.client = boto3.client('dynamodb')
        self.table_name = 'ColorSplashImageIds'
        self.image_id_attribute_key = 'ImageId'
        self.full_url_attribute_key = 'FullURL'
        self.regular_url_attribute_key = 'RegularURL'
        self.small_url_attribute_key = 'SmallURL'
        self.thumbnail_url_attribute_key = 'ThumbnailURL'

    def get_key(self, key):
        '''Given a ImageId key, retrieve the remaing attributes and deserialize it into native 
        python types'''

        if key is None:
            raise TypeError("Parameter 'key' cannot be of type None")

        try:
            response = self.client.get_item(
                Key={
                    self.image_id_attribute_key: {
                        'S': key
                    }
                },
                TableName=self.table_name
            )

            if 'Item' not in response:
                raise KeyError("No such key: " + key)
            else:
                return self.deserialize_urls(response['Item'])
        
        except Exception as e:
            logging.error(e)
            raise

    def deserialize_urls(self, serialized_items):
        ''' Given a row of the table, deserialize the DynamoDB specific format into python native types'''
        logging.debug(serialized_items)

        urls = {}
        for key, value in serialized_items.items():
            urls[key] = self.deserialize_string(value)

        return urls

    def deserialize_string(self, string_attribute):
        ''' Given a string dynamodb row attribute, deserialize it'''
        if 'S' not in string_attribute:
            raise ValueError("Missing String('S') data in attribute")  

        return string_attribute['S']


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['colorsplash_common']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.19.3,<2.0.0']

setup_kwargs = {
    'name': 'colorsplash-common',
    'version': '0.3.1',
    'description': 'A common set of classes for the ColorSplash project python components',
    'long_description': "# colorsplash-common\n\n## Description\n\ncolorsplash-common (CSC) is a component of the overarching ColorSplash web application that can be found at https://thurau.io/colorsplash/. ColorSplash allows users to browse royalty free images that have colors within a certain Euclidean distance of a provided HEX code. CSC is a python library hosted on PyPi to allow common features to be included as dependencies within the other python componenets.\n\nYou can see other components of this project in the following Github repos\n- [ColorSplashPhotoRetrieval](https://github.com/DanielThurau/ColorSplashPhotoRetrieval)\n- [ColorSplashPhotoProcessor](https://github.com/DanielThurau/ColorSplashPhotoProcessor)\n- [ColorSplashColorDetector](https://github.com/DanielThurau/ColorSplashColorDetector)\n- [thurau.io](https://github.com/DanielThurau/thurau.io)\n\n## Motivation\n\nA friend was facing issues when trying to create social media posts for an ecommerce company we recently launched. She had developed a branding guide and had chosen what colors she wanted to include in the website, logos, and eventual marketing material. But when it was time to make marketing posts, trying to apply that style guide was difficult. For all the tools on the internet she used, none were able to query royalty free images that were close to the HEX color codes she had selected. This project was born to remedy this issue.\n\nI wanted to provide a clean minimal interface on a website that would have a form for a HEX code, and query a REST API that would return royalty free images that had a subset of colors within close to the original HEX code.\n\n## Tech Used\n\nCSC is a very simple python library that tightly couples integration with the various DynamoDB tables in the project. It uses boto3 as the client library.\n\n## Installation\n\n### Cloning The Project\n\nYou can either fork the repo or clone it directly with\n\n```shell\n$ git clone https://github.com/DanielThurau/colorsplash-common.git\n$ cd colorsplash-common\n```\n\n### Developing\n\nThis project uses [poetry](https://python-poetry.org/) to build, test, and publish.\n\nTo install dependencies run\n\n```shell\n$ poetry install\n```\n\nTo run the unit tests run\n\n```shell\n$ poetry run pytest\n```\n\n\n## Usage\n\nCSC is hosted on PyPi so you can use any tool that integrates with PyPi to bring it in as a dependency. The easiest way is with pip. For example\n\n```shell\n$ pip install colorsplash-common\n```\n\n// TODO link a Github gist showing the use of the library\n\n## Contribute\n\nIf you'd like to contribute, fork the project and submit a Pull Request.\n\n## License\n\nSee LICENSE.md\n\n> MIT License\n>\n> Copyright (c) 2021 Daniel Thurau",
    'author': 'Daniel Thurau',
    'author_email': 'daniel.n.thurau@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://thurau.io/colorsplash/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
